package servidor;

import java.io.PrintStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.InputMismatchException;
import java.util.Scanner;
import java.util.concurrent.locks.ReentrantLock;


public class Controlador extends Thread
{
	private static ReentrantLock serverLock = new ReentrantLock();
	private ServidorControlador owner = null;
	private ServidorRemoto remote = null;
	private PrintStream scn = null;
	private Scanner kbd = null;
	private int privilege;
	
	public Controlador(ServidorControlador sc, int p)
	{
		owner = sc;
		privilege = p;
	}
	
	public void run()
	{

		String input = "s";
		while (input.length() > 0)
		{
			switch (input.charAt(0))
			{
			case 'q':
			case 'Q':
				owner.exit();
				return;
			case 's': //Estado
			case 'S':
				scn.println(owner.getServer().status());
				scn.println("Puerto " + owner.getServer().getPort());
				scn.println("IP   " + getIP());
				if (owner.getServer().isIgnoring())
					scn.println("Ignorando nuevas conexiones.");
				else
					scn.println("Aceptando nuevas conexiones.");
				if (owner.getServer().isClosingWhenEmpty())
					scn.println("Se cerrar cuando no estn jugando al juego.");
				scn.flush();
				break;
			case 'g': //Lista de juegos
			case 'G':
				scn.println(owner.getServer().status());
				scn.println(owner.getServer().games());
				scn.flush();
				break;
			case 'c': // Lista de controladores remotos
			case 'C':
				scn.println(owner.getRemote(this));
				break;
			case 'r': //Reanudar
			case 'R':
				if (privilege > 1) break;
				
				scn.print("Ests seguro? Esto desconectar todos los juegos. (y / n) ");
				scn.flush();
				String yn = kbd.next();
				if (yn.length() != 0 && yn.charAt(0) == 'y')
				{
					int port = owner.getServer().getPort();
					serverLock.lock();
					{
						owner.getServer().close();
						owner.setServer(new Servidor(port, owner));
					}
					serverLock.unlock();
					scn.println("Reiniciando.");
					scn.flush();
				}
				break;
			case 'p': //Puerto #####
			case 'P':
				if (privilege > 1) break;
				
				int port = readPort();
				owner.getServer().setPort(port);
				scn.println("Conjunto de puertos " + port);
				scn.flush();
				break;
			case 'i': //Ignorar (conmutacin)
			case 'I':
				if (privilege > 1) break;
				
				owner.getServer().ignore(!owner.getServer().isIgnoring());
				if (owner.getServer().isIgnoring())
					scn.println("Ignorar");
				else
					scn.println("No Ignorar");
				scn.flush();
				break;
			case 'x': //Se cierran cuando se estn jugando a los juegos (conmutacin)
			case 'X':
				if (privilege > 1) break;
				
				owner.getServer().closeWhenEmpty(!owner.getServer().isClosingWhenEmpty());
				if (owner.getServer().isClosingWhenEmpty())
					scn.println("Cerrada cuando se vaca");
				else
					scn.println("Cerrada cuando no se vaca");
				scn.flush();
				break;
			case 'a': //Activar la entrada remota
			case 'A':
				if (privilege > 1) break;
				
				int priv;
				String p = kbd.next();
				int remotePort = readPort();
				String password = kbd.next();
				switch (p.charAt(0))
				{
				case 'a':
				case 'A':
				case '1':
					priv = 1;
					p = "Admin";
					break;
				case '2':
					priv = 2;
					p = "level 2";
					break;
				case '3':
					priv = 3;
					p = "level 3";
					break;
				case '4':
					priv = 4;
					p = "level 4";
					break;
				case 'g':
				case 'G':
				case '5':
				default:
					priv = 5;
					p = "Invitado";
					break;
				}
				if (spawn(remotePort, password, priv))
				{
					scn.println("Aceptando " + p + " conexiones en puerto " + remotePort);
					scn.println("Password: " + password);
				}
				break;
			case 'd': //Desconexion remota de conexion(es)
			case 'D':
				if (privilege > 0) break;
				owner.disconnectAll();
				break;
			case 'l': //Cierre de sesin (a distancia)
			case 'L':
				if (remote != null)
				{
					scn.println("Cerrando sesin...");
					scn.flush();
					logout();
					return;
				}
				break;
			case 'h':
			case 'H':
				scn.println("h - Ayuda");
				scn.println("s - Estado");
				scn.println("g - Lista de juegos");
				scn.println("c - (Solo local) Lista de controles");
				scn.println("r - (Admin) Reinicio (cierra todos los juegos)");
				scn.println("p - (Admin) Cambio de puerto");
				scn.println("\tusage: p port\n\texample: p 12345");
				scn.println("i - (Admin) (toggle) Ignorando nuevas conexiones (Cierra juegos que no han comenzado)");
				scn.println("x - (Admin) (toggle) Cerrar cuando no hay juegos que se estn reproduciendo");
				scn.println("a - (Admin) Activa entrada remota");
				scn.println("\tusage: a [guest|admin] port password\n\texample: a guest 12345 yxQ!7%a");
				scn.println("d - (Solo local) Desactiva todas las entradas remotas");
				scn.println("l - (Solo remoto) Cierre de sesin");
				scn.println("Ver documentacin para ms detalles.");
				scn.flush();
				break;
			default:
				scn.println("Tipo de ayuda para obtener instrucciones.");
				break;
			}
			
			try
			{
				input = "";
				input = kbd.next();
			}
			catch (Exception e)
			{
				//Conexiones perdidas
				if (remote != null)
					logout();
				return;
			}
		}
	}
	
	public void connect(Scanner in, PrintStream out)
	{
		kbd = in;
		scn = out;
		if (remote != null)
		{
			owner.print("Conexin remota establecida: " + remote.getIP() + "\n");
			spawn(remote.getPort(), remote.getPass(), privilege);
		}
	}
	
	public void setRemote(ServidorRemoto rc)
	{
		remote = rc;
	}
	
	public ServidorRemoto getRemote()
	{
		return remote;
	}
	
	public void logout()
	{
		disconnect();
		owner.remove(this);
	}
	
	public int getPrivilege()
	{
		return privilege;
	}
	
	public synchronized void disconnect()
	{
		if (remote != null)
		{
			if (scn != null) scn.close();
			if (kbd != null) kbd.close();
			remote.disconnect();
		}
	}
	
	public void print(String e)
	{
		if (scn != null)
		{
			scn.println(e);
			scn.flush();
		}
	}
	
	private boolean spawn(int port, String password, int priv)
	{
		Controlador c = new Controlador(owner, priv);
		c.setRemote(new ServidorRemoto(c, password));
		if (c.getRemote().init(port, this))
		{
			scn.flush();
			owner.add(c);
			c.getRemote().start();
			return true;
		}
		return false;
	}
	
	private String getIP()
	{
		try
		{
			return ""+InetAddress.getLocalHost();
		}
		catch (UnknownHostException e)
		{
			return "unknown";
		}
	}
	
	private int readPort()
	{
		int p;
		while (true)
		{
			try
			{
				p = kbd.nextInt();
			}
			catch (InputMismatchException e)
			{
				kbd.nextLine();
				scn.print("Introduzca puerto: ");
				scn.flush();
				continue;
			}
			break;
		}
		
		return p;
	}
}