package servidor;

import java.io.IOException;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Scanner;


public class ServidorRemoto 
{
	private Controlador controller = null;
	private ServerSocket ss = null;
	private Socket sock = null;
	private Scanner in = null;
	private PrintStream out = null;
	private Thread listen = null;
	private String password = "";
	private int port;
	private boolean closing = false;
	
	public ServidorRemoto(Controlador sc, String pass)	
	{
		controller = sc;
		password = pass;
	}
	
	public boolean init(int p, Controlador notify)
	{
		port = p;
		try
		{
			ss = new ServerSocket(port);
		}
		catch (IOException e)
		{
			if (notify != null)
				notify.print("Puerto no disponible.");
			controller.disconnect();
			return false;
		}
		return true;
	}
	
	public void start()
	{
		listen = new Thread()
		{
			public void run()
			{
				boolean valid = false;;
				do try
				{
					if (sock != null)
						sock.close();
					
					sock = ss.accept();

					in = new Scanner(sock.getInputStream());
					out = new PrintStream(sock.getOutputStream());
					valid = validate();
					if (!valid)
						out.println("Password invalido.");
				}
				catch (Exception e)
				{
					if (!closing)
					{
						controller.disconnect();
						return;
					}
				} while (!valid && !closing);
				
				try
				{
					ss.close();
					ss = null;
					controller.connect(in, out);
					controller.start();
				}
				catch (Exception e)
				{
					if (!closing)
					{
						controller.disconnect();
						return;
					}
				}
			}
		};
		
		listen.start();
	}
	
	public int getPort()
	{
		return port;
	}
	
	public void setPort(int p)
	{
		port = p;
	}
	
	public String getPass()
	{
		return password;
	}
	
	public void setPass(String p)
	{
		password = p;
	}
	
	public String getIP()
	{
		if (sock != null && sock.isConnected())
			return sock.getInetAddress().getHostAddress();
		else
			return "Sin conexin";
	}

	public void disconnect()
	{
		closing = true;
		
		if (ss != null) try
		{
			ss.close();
		}
		catch (IOException e){}
		finally
		{
			ss = null;
		}
		
		if (sock != null) try
		{
			sock.close();
		}
		catch (IOException e){}
		finally
		{
			sock = null;
		}
	}
	
	public String status()
	{
		if (ss != null)
			return "Aceptando controlador remoto en el puerto " + ss.getLocalPort();
		if (sock.isConnected())
			return "Conectado a controlador remoto " + sock.getInetAddress().getHostAddress();
		return "Error en el controlador remoto!";
	}
	
	private boolean validate()
	{
		String salt = Hash.Sha1(""+System.currentTimeMillis());
		out.println(salt);
		String pass = in.next();
		if (Hash.Sha1(salt+password).equals(pass))
			return true;
		return false;
	}
}
